library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity TwoLimitCounter is
	generic (
		NBits : integer := 4;
		Limit1 : integer := 3;
		Limit2 : integer := 2
	);
	port (
		Reset       : in std_logic;
		Enable      : in std_logic;
		Clock       : in std_logic;
		Terminated1 : out std_logic;
		Terminated2 : out std_logic
	);
end TwoLimitCounter;

architecture Architecture1 of TwoLimitCounter is
	component Adder is
		generic (
			NBits : integer := 4
		);
		port (
			A : in std_logic_vector((NBits - 1) downto 0);
			B : in std_logic_vector((NBits - 1) downto 0);
			Y : out std_logic_vector((NBits - 1) downto 0)
		);
	end component;
	signal QBus : std_logic_vector((NBits - 1) downto 0);
	signal DBus : std_logic_vector((NBits - 1) downto 0);
	signal PreDBus : std_logic_vector((NBits - 1) downto 0);
begin
	-- D flip-flop with synchronous reset
	process (Clock, Reset)
	begin
		if (Clock'event and (Clock = '1')) then
			if (Reset = '1') then
				QBus <= (others => '0');
			else
				QBus <= DBus;
			end if;
		end if;
	end process;

	-- enable logic
	DBus <= PreDBus when (Enable = '1') else QBus;
	-- next state logic
	A : Adder generic map (
		NBits => NBits
	)
	port map (
		A => QBus,
		B => std_logic_vector(to_unsigned(1, NBits)),
		Y => PreDBus
    );
	--PreDBus <= QBus + 1;
	-- output logic
	Terminated1 <= '1' when (QBus = std_logic_vector(to_unsigned(Limit1, NBits))) else '0';
	Terminated2 <= '1' when (QBus = std_logic_vector(to_unsigned(Limit2, NBits))) else '0';
end Architecture1;
